@@ -5,7 +5,6 @@ package Locale::Babelfish::Phrase::Parser;
 use utf8;
 use strict;
 use warnings;
-use feature "state";
 
 use Locale::Babelfish::Phrase::Literal ();
 use Locale::Babelfish::Phrase::Variable ();
@@ -14,7 +13,7 @@ use Locale::Babelfish::Phrase::PluralFormsParser ();
 
 use parent qw( Locale::Babelfish::Phrase::ParserBase );
 
-our $VERSION = '1.000001'; # VERSION
+our $VERSION = '2.001'; # VERSION
 
 __PACKAGE__->mk_accessors( qw( locale mode pieces escape pf0 ) );
 
@@ -70,7 +69,7 @@ sub parse {
     $self->SUPER::parse( $phrase );
     $self->locale( $locale )  if $locale;
 
-    state $plurals_parser = Locale::Babelfish::Phrase::PluralFormsParser->new();
+    my $plurals_parser = Locale::Babelfish::Phrase::PluralFormsParser->new();
 
     while ( 1 ) {
         my $char = $self->to_next_char;
@@ -187,7 +186,7 @@ Locale::Babelfish::Phrase::Parser - Babelfish syntax parser.
 
 =head1 VERSION
 
-version 1.000001
+version 2.001
 
 =head1 METHODS
 
@@ -227,13 +226,17 @@ Igor Mironov <grif@cpan.org>
 
 =item *
 
+Victor Efimov <efimov@reg.ru>
+
+=item *
+
 REG.RU LLC
 
 =back
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Akzhan Abdulin.
+This software is Copyright (c) 2014 by REG.RU LLC.
 
 This is free software, licensed under:
 
